library angular2.src.compiler.view_compiler.compile_view;

import "package:angular2/src/facade/lang.dart" show isPresent, isBlank;
import "package:angular2/src/facade/collection.dart"
    show ListWrapper, StringMapWrapper, MapWrapper;
import "../output/output_ast.dart" as o;
import "constants.dart" show EventHandlerVars;
import "compile_query.dart"
    show CompileQuery, createQueryList, addQueryToTokenMap;
import "expression_converter.dart" show NameResolver;
import "compile_element.dart" show CompileElement, CompileNode;
import "compile_method.dart" show CompileMethod;
import "compile_pipe.dart" show CompilePipe;
import "package:angular2/src/core/linker/view_type.dart" show ViewType;
import "../compile_metadata.dart"
    show
        CompileDirectiveMetadata,
        CompilePipeMetadata,
        CompileIdentifierMetadata,
        CompileTokenMap;
import "util.dart"
    show
        getViewFactoryName,
        injectFromViewParentInjector,
        createDiTokenExpression,
        getPropertyInView,
        createPureProxy;
import "../config.dart" show CompilerConfig;
import "compile_binding.dart" show CompileBinding;

class CompileView implements NameResolver {
  CompileDirectiveMetadata component;
  CompilerConfig genConfig;
  List<CompilePipeMetadata> pipeMetas;
  o.Expression styles;
  num viewIndex;
  CompileElement declarationElement;
  List<List<String>> templateVariableBindings;
  ViewType viewType;
  CompileTokenMap<List<CompileQuery>> viewQueries;
  List<CompileNode> nodes = [];
  // root nodes or AppElements for ViewContainers
  List<o.Expression> rootNodesOrAppElements = [];
  List<CompileBinding> bindings = [];
  List<o.Statement> classStatements = [];
  CompileMethod createMethod;
  CompileMethod injectorGetMethod;
  CompileMethod updateContentQueriesMethod;
  CompileMethod dirtyParentQueriesMethod;
  CompileMethod updateViewQueriesMethod;
  CompileMethod detectChangesInInputsMethod;
  CompileMethod detectChangesRenderPropertiesMethod;
  CompileMethod afterContentLifecycleCallbacksMethod;
  CompileMethod afterViewLifecycleCallbacksMethod;
  CompileMethod destroyMethod;
  List<o.ClassMethod> eventHandlerMethods = [];
  List<o.ClassField> fields = [];
  List<o.ClassGetter> getters = [];
  List<o.Expression> disposables = [];
  List<o.Expression> subscriptions = [];
  CompileView componentView;
  var purePipes = new Map<String, CompilePipe>();
  List<CompilePipe> pipes = [];
  var variables = new Map<String, o.Expression>();
  String className;
  o.Type classType;
  o.ReadVarExpr viewFactory;
  var literalArrayCount = 0;
  var literalMapCount = 0;
  var pipeCount = 0;
  CompileView(this.component, this.genConfig, this.pipeMetas, this.styles,
      this.viewIndex, this.declarationElement, this.templateVariableBindings) {
    this.createMethod = new CompileMethod(this);
    this.injectorGetMethod = new CompileMethod(this);
    this.updateContentQueriesMethod = new CompileMethod(this);
    this.dirtyParentQueriesMethod = new CompileMethod(this);
    this.updateViewQueriesMethod = new CompileMethod(this);
    this.detectChangesInInputsMethod = new CompileMethod(this);
    this.detectChangesRenderPropertiesMethod = new CompileMethod(this);
    this.afterContentLifecycleCallbacksMethod = new CompileMethod(this);
    this.afterViewLifecycleCallbacksMethod = new CompileMethod(this);
    this.destroyMethod = new CompileMethod(this);
    this.viewType = getViewType(component, viewIndex);
    this.className = '''_View_${ component . type . name}${ viewIndex}''';
    this.classType =
        o.importType(new CompileIdentifierMetadata(name: this.className));
    this.viewFactory = o.variable(getViewFactoryName(component, viewIndex));
    if (identical(this.viewType, ViewType.COMPONENT) ||
        identical(this.viewType, ViewType.HOST)) {
      this.componentView = this;
    } else {
      this.componentView = this.declarationElement.view.componentView;
    }
    var viewQueries = new CompileTokenMap<List<CompileQuery>>();
    if (identical(this.viewType, ViewType.COMPONENT)) {
      var directiveInstance = o.THIS_EXPR.prop("context");
      ListWrapper.forEachWithIndex(this.component.viewQueries,
          (queryMeta, queryIndex) {
        var propName =
            '''_viewQuery_${ queryMeta . selectors [ 0 ] . name}_${ queryIndex}''';
        var queryList =
            createQueryList(queryMeta, directiveInstance, propName, this);
        var query =
            new CompileQuery(queryMeta, queryList, directiveInstance, this);
        addQueryToTokenMap(viewQueries, query);
      });
      var constructorViewQueryCount = 0;
      this.component.type.diDeps.forEach((dep) {
        if (isPresent(dep.viewQuery)) {
          var queryList = o.THIS_EXPR
              .prop("declarationAppElement")
              .prop("componentConstructorViewQueries")
              .key(o.literal(constructorViewQueryCount++));
          var query = new CompileQuery(dep.viewQuery, queryList, null, this);
          addQueryToTokenMap(viewQueries, query);
        }
      });
    }
    this.viewQueries = viewQueries;
    templateVariableBindings.forEach((entry) {
      this.variables[entry[1]] =
          o.THIS_EXPR.prop("locals").key(o.literal(entry[0]));
    });
    if (!this.declarationElement.isNull()) {
      this.declarationElement.setEmbeddedView(this);
    }
  }
  o.Expression callPipe(
      String name, o.Expression input, List<o.Expression> args) {
    var compView = this.componentView;
    var pipe = compView.purePipes[name];
    if (isBlank(pipe)) {
      pipe = new CompilePipe(compView, name);
      if (pipe.pure) {
        compView.purePipes[name] = pipe;
      }
      compView.pipes.add(pipe);
    }
    return pipe.call(this, (new List.from([input])..addAll(args)));
  }

  o.Expression getVariable(String name) {
    if (name == EventHandlerVars.event.name) {
      return EventHandlerVars.event;
    }
    CompileView currView = this;
    var result = currView.variables[name];
    while (isBlank(result) && isPresent(currView.declarationElement.view)) {
      currView = currView.declarationElement.view;
      result = currView.variables[name];
    }
    if (isPresent(result)) {
      return getPropertyInView(result, this, currView);
    } else {
      return null;
    }
  }

  o.Expression createLiteralArray(List<o.Expression> values) {
    var proxyExpr =
        o.THIS_EXPR.prop('''_arr_${ this . literalArrayCount ++}''');
    List<o.FnParam> proxyParams = [];
    List<o.Expression> proxyReturnEntries = [];
    for (var i = 0; i < values.length; i++) {
      var paramName = '''p${ i}''';
      proxyParams.add(new o.FnParam(paramName));
      proxyReturnEntries.add(o.variable(paramName));
    }
    createPureProxy(
        o.fn(proxyParams,
            [new o.ReturnStatement(o.literalArr(proxyReturnEntries))]),
        values.length,
        proxyExpr,
        this);
    return proxyExpr.callFn(values);
  }

  o.Expression createLiteralMap(
      List<List<dynamic /* String | o . Expression */ >> entries) {
    var proxyExpr = o.THIS_EXPR.prop('''_map_${ this . literalMapCount ++}''');
    List<o.FnParam> proxyParams = [];
    List<List<dynamic /* String | o . Expression */ >> proxyReturnEntries = [];
    List<o.Expression> values = [];
    for (var i = 0; i < entries.length; i++) {
      var paramName = '''p${ i}''';
      proxyParams.add(new o.FnParam(paramName));
      proxyReturnEntries.add([entries[i][0], o.variable(paramName)]);
      values.add((entries[i][1] as o.Expression));
    }
    createPureProxy(
        o.fn(proxyParams,
            [new o.ReturnStatement(o.literalMap(proxyReturnEntries))]),
        entries.length,
        proxyExpr,
        this);
    return proxyExpr.callFn(values);
  }

  afterNodes() {
    this.pipes.forEach((pipe) => pipe.create());
    this.viewQueries.values().forEach((queries) => queries
        .forEach((query) => query.afterChildren(this.updateViewQueriesMethod)));
  }
}

ViewType getViewType(
    CompileDirectiveMetadata component, num embeddedTemplateIndex) {
  if (embeddedTemplateIndex > 0) {
    return ViewType.EMBEDDED;
  } else if (component.type.isHost) {
    return ViewType.HOST;
  } else {
    return ViewType.COMPONENT;
  }
}
